/**
 * Authentication utilities for LoadFocus API
 */
const configManager = require('../../config/config-manager');

/**
 * Get authentication credentials
 * @param {Object} options - Options for retrieving credentials
 * @param {string} options.apiKey - API key (optional, will use config if not provided)
 * @param {string} options.teamId - Team ID (optional, will use config if not provided)
 * @returns {Object} Authentication credentials
 */
function getCredentials(options = {}) {
    const apiKey = options.apiKey || configManager.get('apiKey');
    const teamId = options.teamId || configManager.get('teamId');

    if (!apiKey) {
        throw new Error('API key is required. Please set it via the config command or provide it in options.');
    }

    if (!teamId) {
        throw new Error('Team ID is required. Please set it via the config command or provide it in options.');
    }

    return {
        apiKey,
        teamId
    };
}

/**
 * Create user identity string in the format needed by the API
 * @param {string} teamId - Team ID
 * @param {string} userId - User ID (optional)
 * @returns {string} User identity JSON string
 */
function createUserIdentity(teamId, userId = null) {
    const identity = {
        team: teamId
    };

    if (userId) {
        identity.user = userId;
    }

    return JSON.stringify(identity);
}

/**
 * Validate API key format
 * @param {string} apiKey - API key to validate
 * @returns {boolean} True if valid format
 */
function isValidApiKey(apiKey) {
    // Simple validation - implement according to your API key format
    return typeof apiKey === 'string' && apiKey.length >= 10;
}

/**
 * Validate Team ID format
 * @param {string} teamId - Team ID to validate
 * @returns {boolean} True if valid format
 */
function isValidTeamId(teamId) {
    // Simple validation - implement according to your team ID format
    return typeof teamId === 'string' && teamId.length > 0;
}

module.exports = {
    getCredentials,
    createUserIdentity,
    isValidApiKey,
    isValidTeamId
};