/**
 * Base client for all LoadFocus API services
 */
const HttpClient = require('./http-client');

class BaseClient {
    /**
     * Create a new BaseClient
     * @param {Object} options - Client configuration
     * @param {string} options.apiKey - API key for authentication
     * @param {string} options.teamId - Team ID
     * @param {string} options.apiUrl - API base URL (optional)
     */
    constructor(options = {}) {
        this.apiKey = options.apiKey;
        this.teamId = options.teamId;
        this.apiUrl = options.apiUrl || 'https://loadfocus.com';

        this.httpClient = new HttpClient({
            baseURL: this.apiUrl,
            headers: {
                'loadfocus-auth': this.apiKey,
                'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
        });
    }

    /**
     * Get user identity string in the format needed by the API
     * @returns {string} User identity string
     */
    getUserIdentity() {
        // Create the user identity JSON string as expected by the API
        const userIdentity = {
            user: this.userId,
            team: this.teamId
        };

        return JSON.stringify(userIdentity);
    }

    /**
     * Make an authenticated GET request
     * @param {string} path - API endpoint path
     * @param {Object} params - Query parameters (optional)
     * @returns {Promise<Object>} - Response data
     */
    async get(path, params = {}) {
        return this.httpClient.get(path, { params });
    }

    /**
     * Make an authenticated POST request
     * @param {string} path - API endpoint path
     * @param {Object} data - Request payload
     * @param {Object} config - Additional axios config (optional)
     * @returns {Promise<Object>} - Response data
     */
    async post(path, data, config = {}) {
        return this.httpClient.post(path, data, config);
    }

    /**
     * Make a form POST request
     * @param {string} path - API endpoint path
     * @param {Object} formData - Form data
     * @returns {Promise<Object>} - Response data
     */
    async postForm(path, formData) {
        return this.httpClient.postForm(path, formData);
    }
}

module.exports = BaseClient;