/**
 * HTTP client wrapper around axios
 */
const axios = require('axios');
const FormData = require('form-data');

class HttpClient {
    /**
     * Create a new HTTP client
     * @param {Object} config - Axios configuration
     */
    constructor(config = {}) {
        this.client = axios.create(config);

        // Add response interceptor for error handling
        this.client.interceptors.response.use(
            response => response.data,
            error => {
                // Transform error for better client-side handling
                if (error.response) {
                    // The request was made and the server responded with a status code
                    // that falls out of the range of 2xx
                    const serverError = new Error(`Server error: ${error.response.status}`);
                    serverError.status = error.response.status;
                    serverError.data = error.response.data;
                    return Promise.reject(serverError);
                } else if (error.request) {
                    // The request was made but no response was received
                    const networkError = new Error('Network error: No response received');
                    networkError.request = error.request;
                    return Promise.reject(networkError);
                } else {
                    // Something happened in setting up the request that triggered an Error
                    const configError = new Error(`Request error: ${error.message}`);
                    return Promise.reject(configError);
                }
            }
        );
    }

    /**
     * Make a GET request
     * @param {string} url - The URL to request
     * @param {Object} config - Axios request configuration
     * @returns {Promise<Object>} - Response data
     */
    async get(url, config = {}) {
        return this.client.get(url, config);
    }

    /**
     * Make a POST request
     * @param {string} url - The URL to request
     * @param {Object} data - The data to send
     * @param {Object} config - Axios request configuration
     * @returns {Promise<Object>} - Response data
     */
    async post(url, data, config = {}) {
        console.log(url);
        return this.client.post(url, data, config);
    }

    /**
     * Make a POST request with form data
     * @param {string} url - The URL to request
     * @param {Object} data - Key-value pairs for the form
     * @returns {Promise<Object>} - Response data
     */
    async postForm(url, data) {
        const formData = new FormData();

        // Add all data to the form
        Object.entries(data).forEach(([key, value]) => {
            // Handle arrays
            if (Array.isArray(value)) {
                value.forEach(item => {
                    formData.append(`${key}[]`, item);
                });
            } else if (value !== undefined && value !== null) {
                formData.append(key, value);
            }
        });

        console.log(url);
        return this.client.post(url, formData, {
            headers: {
                ...formData.getHeaders()
            }
        });
    }

    /**
     * Make a PUT request
     * @param {string} url - The URL to request
     * @param {Object} data - The data to send
     * @param {Object} config - Axios request configuration
     * @returns {Promise<Object>} - Response data
     */
    async put(url, data, config = {}) {
        return this.client.put(url, data, config);
    }

    /**
     * Make a DELETE request
     * @param {string} url - The URL to request
     * @param {Object} config - Axios request configuration
     * @returns {Promise<Object>} - Response data
     */
    async delete(url, config = {}) {
        return this.client.delete(url, config);
    }
}

module.exports = HttpClient;