/**
 * JMeter API client
 */
const BaseClient = require('../../core/base-client');
const auth = require('../../core/auth');

class JMeterClient extends BaseClient {
    /**
     * Create a new JMeter API client
     * @param {Object} options - Client configuration
     * @param {string} options.apiKey - API key for authentication
     * @param {string} options.teamId - Team ID
     * @param {string} options.apiUrl - API base URL (optional)
     */
    constructor(options = {}) {
        const credentials = auth.getCredentials(options);

        super({
            apiKey: credentials.apiKey,
            teamId: credentials.teamId,
            apiUrl: options.apiUrl
        });

        this.servicePath = '/jmeter';
    }

    /**
     * Get a list of recent test runs
     * @param {Object} options - Query options
     * @param {string} options.term - Search term (optional)
     * @returns {Promise<Object>} API response
     */
    async getRuns(options = {}) {
        return this.get(`${this.servicePath}/runs`, {
            term: options.term || ''
        });
    }

    /**
     * Get test status
     * @param {Object} options - Request options
     * @param {string} options.testrunname - Test run name
     * @param {string} options.testrunid - Test run ID
     * @returns {Promise<Object>} API response
     */
    async getStatus(options) {
        return this.get(`/api/v1/jmetertests/state`, {
            testrunname: options.testrunname,
            testrunid: options.testrunid
        });
    }

    /**
     * Get test configuration
     * @param {Object} options - Request options
     * @param {string} options.testrunname - Test run name
     * @returns {Promise<Object>} API response
     */
    async getConfig(options) {
        return this.get(`${this.servicePath}/retrieveconfig`, {
            testrunname: options.testrunname
        });
    }

    /**
     * Get test labels
     * @param {Object} options - Request options
     * @param {string} options.testrunname - Test run name
     * @param {string} options.testrunid - Test run ID
     * @returns {Promise<Object>} API response
     */
    async getLabels(options) {
        // Create the URL with query parameters
        const url = `/api/v1/jmetertests/labels?testrunname=${encodeURIComponent(options.testrunname)}&testrunid=${encodeURIComponent(options.testrunid)}`;

        // Make a POST request with an empty body
        return this.post(url, {});
    }

    /**
     * Get test results
     * @param {Object} options - Request options
     * @param {string} options.testrunname - Test run name
     * @param {string} options.testrunid - Test run ID
     * @param {number} options.from - Start timestamp
     * @param {number} options.to - End timestamp
     * @param {string} options.granularity - Result granularity
     * @param {Array} options.include - Fields to include
     * @param {Array} options.filter - Labels to filter
     * @param {Array} options.sortasc - Fields to sort ascending
     * @param {Array} options.sortdesc - Fields to sort descending
     * @param {number} options.batchsize - Batch size
     * @param {number} options.skipsize - Skip size
     * @returns {Promise<Object>} API response
     */
    async getResults(options) {
        const formData = {
            testrunname: options.testrunname,
            testrunid: options.testrunid,
            teststarttime: options.from,
            teststoptime: options.to,
            granularity: options.granularity || '1s',
            batchsize: options.batchsize || 1000,
            skipsize: options.skipsize || 0
        };

        // Include fields
        const include = options.include || [
            'timestamp', 'mean', 'median', 'p90', 'p95', 'p99',
            'min', 'max', 'ep', 'rt', 'l', 'ct', 'ec', 'ecnt',
            'th', 'kbrs', 'kbss', 'sd', 'label', 'url', 'statscnt', 'no'
        ];

        formData['include[]'] = include;

        // Add optional parameters if provided
        if (options.filter) {
            formData['filter[]'] = options.filter;
        }

        if (options.sortasc) {
            formData['sortasc[]'] = options.sortasc;
        }

        if (options.sortdesc) {
            formData['sortdesc[]'] = options.sortdesc;
        }

        return this.postForm(`/api/v1/jmetertests/aggregate/results-noform`, formData);
    }

    /**
     * Execute a JMeter test
     * @param {Object} options - Request options
     * @param {string} options.testrunname - Test run name
     * @param {string} options.testrunid - Test run ID (optional)
     * @returns {Promise<Object>} API response
     */
    async execute(options) {
        const formData = {
            testrunname: options.testrunname,
            testrunid: options.testrunid || ''
        };

        return this.postForm(`/api/v1/jmetertests/execute`, formData);
    }

    /**
     * Get current plan information
     * @returns {Promise<Object>} API response with plan details
     */
    async getPlan() {
        return this.get(`${this.servicePath}/plan`);
    }
}

module.exports = JMeterClient;