/**
 * JMeter test configuration command
 */
const chalk = require('chalk');
const ora = require('ora');
const configEndpoint = require('../endpoints/config');
const JMeterClient = require('../client');
const configManager = require('../../../../config/config-manager');

/**
 * Get configuration of a JMeter test
 * @param {Object} options - Command options
 */
async function config(options) {
    // Check if configuration is set up
    if (!configManager.isConfigured()) {
        console.log(chalk.red('Error: Missing required configuration.'));
        const missing = configManager.getMissingKeys();
        missing.forEach(key => {
            console.log(chalk.yellow(`  - ${key}`));
        });
        console.log(chalk.yellow('\nUse "loadfocus-api config set" to configure these values.'));
        return;
    }

    // Create spinner
    const spinner = ora('Fetching test configuration...').start();

    try {
        // Create client
        const client = new JMeterClient();

        // Get test configuration
        const result = await configEndpoint(client, {
            testrunname: options.name
        });

        spinner.succeed('Test configuration retrieved successfully');

        // Display configuration
        console.log();
        console.log(chalk.cyan('Test Configuration:'));
        console.log(chalk.cyan('------------------'));
        console.log(`Test Name: ${chalk.green(options.name)}`);
        console.log();

        // Pretty print the configuration
        console.log(JSON.stringify(result, null, 2));

        console.log();
        console.log(chalk.cyan('Next Steps:'));
        console.log(`- Execute this test: ${chalk.green(`loadfocus-api jmeter execute --name "${options.name}"`)}`);

    } catch (error) {
        spinner.fail('Failed to retrieve test configuration');

        if (error.data) {
            console.log(chalk.red('Error details:'));
            console.log(JSON.stringify(error.data, null, 2));
        } else {
            console.log(chalk.red(`Error: ${error.message}`));
        }
    }
}

module.exports = config;