/**
 * JMeter execute command
 */
const chalk = require('chalk');
const ora = require('ora');
const executeEndpoint = require('../endpoints/execute');
const JMeterClient = require('../client');
const configManager = require('../../../../config/config-manager');

/**
 * Execute a JMeter test
 * @param {Object} options - Command options
 */
async function execute(options) {
    // Check if configuration is set up
    if (!configManager.isConfigured()) {
        console.log(chalk.red('Error: Missing required configuration.'));
        const missing = configManager.getMissingKeys();
        missing.forEach(key => {
            console.log(chalk.yellow(`  - ${key}`));
        });
        console.log(chalk.yellow('\nUse "loadfocus-api config set" to configure these values.'));
        return;
    }

    // Create spinner
    const spinner = ora('Executing JMeter test...').start();

    try {
        // Create client
        const client = new JMeterClient();

        // Execute the test
        const result = await executeEndpoint(client, {
            testrunname: options.name,
            testrunid: options.id
        });

        // Check if the execution was successful
        if (result.success === 'true') {
            spinner.succeed('Test execution started successfully');
        } else {
            spinner.warn('Test execution initiated with warnings');
        }

        // Determine which format to display based on the format option
        const format = options.format ? options.format.toLowerCase() : 'pretty';

        switch (format) {
        case 'json':
            displayJsonFormat(result);
            break;
        case 'raw':
            displayRawFormat(result);
            break;
        case 'pretty':
        default:
            displayPrettyFormat(result);
            break;
        }
    } catch (error) {
        spinner.fail('Failed to execute test');

        if (error.data) {
            console.log(chalk.red('Error details:'));
            console.log(JSON.stringify(error.data, null, 2));
        } else {
            console.log(chalk.red(`Error: ${error.message}`));
        }
    }
}

/**
 * Display execution results in pretty format with colors
 * @param {Object} result - Execution response
 */
function displayPrettyFormat(result) {
    // Display result details
    console.log();
    console.log(chalk.cyan('Test Details:'));
    console.log(chalk.cyan('------------'));
    console.log(`Test Name: ${chalk.green(result.testrunname)}`);

    // Display test ID from response
    if (result.testrunid) {
        console.log(`Test ID: ${chalk.green(result.testrunid)}`);
    }

    // Display any additional message from the API
    if (result.message) {
        console.log(`Message: ${chalk.green(result.message)}`);
    }

    // Display success status
    console.log(`Success: ${result.success === 'true' ? chalk.green('Yes') : chalk.yellow('No')}`);

    console.log();
    console.log(chalk.cyan('Next Steps:'));
    console.log(`- Check test status: ${chalk.green(`loadfocus-api jmeter status --name "${result.testrunname}" --id "${result.testrunid}"`)}`);
}

/**
 * Display execution results in JSON format
 * @param {Object} result - Execution response
 */
function displayJsonFormat(result) {
    // Create a simplified execution summary
    const executionSummary = {
        testName: result.testrunname,
        testId: result.testrunid,
        success: result.success === 'true',
        message: result.message || null
    };

    console.log();
    console.log(JSON.stringify(executionSummary, null, 2));
}

/**
 * Display the raw API response
 * @param {Object} result - Execution response
 */
function displayRawFormat(result) {
    console.log();
    console.log(JSON.stringify(result, null, 2));
}

module.exports = execute;