/**
 * JMeter labels command
 */
const chalk = require('chalk');
const ora = require('ora');
const Table = require('cli-table3');
const labelsEndpoint = require('../endpoints/labels');
const JMeterClient = require('../client');
const configManager = require('../../../../config/config-manager');

/**
 * Get labels from a JMeter test
 * @param {Object} options - Command options
 */
async function labels(options) {
    // Check if configuration is set up
    if (!configManager.isConfigured()) {
        console.log(chalk.red('Error: Missing required configuration.'));
        const missing = configManager.getMissingKeys();
        missing.forEach(key => {
            console.log(chalk.yellow(`  - ${key}`));
        });
        console.log(chalk.yellow('\nUse "loadfocus-api config set" to configure these values.'));
        return;
    }

    // Create spinner
    const spinner = ora('Fetching test labels...').start();

    try {
        // Create client
        const client = new JMeterClient();

        // Get test labels
        const result = await labelsEndpoint(client, {
            testrunname: options.name,
            testrunid: options.id
        });

        spinner.succeed('Test labels retrieved successfully');

        // Check if we have labels
        if (!result || !result.length) {
            if (options.format === 'json') {
                console.log(JSON.stringify([]));
            } else {
                console.log(chalk.yellow('\nNo labels found for this test.'));
            }
            return;
        }

        // Determine which format to display based on the format option
        const format = options.format ? options.format.toLowerCase() : 'pretty';

        switch (format) {
        case 'json':
            displayJsonFormat(result);
            break;
        case 'raw':
            displayRawFormat(result);
            break;
        case 'pretty':
        default:
            displayPrettyFormat(result, options);
            break;
        }

    } catch (error) {
        spinner.fail('Failed to retrieve test labels');

        if (error.data) {
            console.log(chalk.red('Error details:'));
            console.log(JSON.stringify(error.data, null, 2));
        } else {
            console.log(chalk.red(`Error: ${error.message}`));
        }
    }
}

/**
 * Display labels in pretty format with colors and table
 * @param {Array} result - Labels response array
 * @param {Object} options - Command options
 */
function displayPrettyFormat(result, options) {
    console.log();
    console.log(chalk.cyan('Test Labels:'));
    console.log(chalk.cyan('------------'));
    console.log(`Test Name: ${chalk.green(options.name)}`);
    console.log(`Test ID: ${chalk.green(options.id)}`);
    console.log();

    // Create table for labels
    const table = new Table({
        head: [
            chalk.cyan('#'),
            chalk.cyan('Label'),
            chalk.cyan('URL')
        ],
        colWidths: [5, 45, 60]
    });

    // Add rows for each label
    result.forEach((labelObj, index) => {
        table.push([
            index + 1,
            labelObj.label || '-',  // Use the label field directly
            labelObj.url || '-'     // Use the url field directly
        ]);
    });

    console.log(table.toString());

    // Display label count
    console.log();
    console.log(`Total Labels: ${chalk.green(result.length)}`);

    console.log();
    console.log(chalk.cyan('Next Steps:'));
    console.log(`- Get test results: ${chalk.green(`loadfocus-api jmeter results --name "${options.name}" --id "${options.id}"`)}`);
}

/**
 * Display labels in JSON format
 * @param {Object} result - Labels result
 */
function displayJsonFormat(result) {
    console.log(JSON.stringify(result, null, 2));
}

/**
 * Display labels in raw format
 * @param {Object} result - Labels result
 */
function displayRawFormat(result) {
    console.log(JSON.stringify(result, null, 2));
}

module.exports = labels;