/**
 * JMeter plan command
 */
const chalk = require('chalk');
const ora = require('ora');
const Table = require('cli-table3');
const planEndpoint = require('../endpoints/plan');
const JMeterClient = require('../client');
const configManager = require('../../../../config/config-manager');

/**
 * Get current plan information
 */
async function plan() {
    // Check if configuration is set up
    if (!configManager.isConfigured()) {
        console.log(chalk.red('Error: Missing required configuration.'));
        const missing = configManager.getMissingKeys();
        missing.forEach(key => {
            console.log(chalk.yellow(`  - ${key}`));
        });
        console.log(chalk.yellow('\nUse "loadfocus-api config set" to configure these values.'));
        return;
    }

    // Create spinner
    const spinner = ora('Fetching plan information...').start();

    try {
        // Create client
        const client = new JMeterClient();

        // Get plan information
        const result = await planEndpoint(client);

        spinner.succeed('Plan information retrieved successfully');

        // Display plan information
        console.log();
        console.log(chalk.cyan('Current Plan:'));
        console.log(chalk.cyan('------------'));

        // Handle different possible formats of the response
        if (result.name) {
            console.log(`Plan Name: ${chalk.green(result.name)}`);

            if (result.paid !== undefined) {
                console.log(`Plan Type: ${result.paid ? chalk.green('Paid') : chalk.yellow('Free')}`);
            }

            if (result.shortName) {
                console.log(`Plan ID: ${chalk.green(result.shortName)}`);
            }

            // Show any additional details
            const detailsToSkip = ['name', 'paid', 'shortName'];
            const additionalDetails = Object.entries(result)
                .filter(([key]) => !detailsToSkip.includes(key));

            if (additionalDetails.length > 0) {
                console.log();
                console.log(chalk.cyan('Plan Details:'));

                const table = new Table({
                    head: [chalk.cyan('Feature'), chalk.cyan('Value')],
                    colWidths: [30, 20]
                });

                additionalDetails.forEach(([key, value]) => {
                    table.push([
                        key.charAt(0).toUpperCase() + key.slice(1).replace(/([A-Z])/g, ' $1'), // Format camelCase to words
                        formatPlanValue(value)
                    ]);
                });

                console.log(table.toString());
            }
        } else {
            // If the response format is different, just show the raw data
            console.log(JSON.stringify(result, null, 2));
        }

    } catch (error) {
        spinner.fail('Failed to retrieve plan information');

        if (error.data) {
            console.log(chalk.red('Error details:'));
            console.log(JSON.stringify(error.data, null, 2));
        } else {
            console.log(chalk.red(`Error: ${error.message}`));
        }
    }
}

/**
 * Format plan value for display
 * @param {*} value - Plan value
 * @returns {string} Formatted value
 */
function formatPlanValue(value) {
    if (value === null || value === undefined) {
        return '-';
    }

    if (typeof value === 'boolean') {
        return value ? '✓' : '✗';
    }

    if (typeof value === 'object') {
        return JSON.stringify(value);
    }

    return value.toString();
}

module.exports = plan;