/**
 * JMeter results command
 */
const chalk = require('chalk');
const ora = require('ora');
const Table = require('cli-table3');
const resultsEndpoint = require('../endpoints/results');
const JMeterClient = require('../client');
const configManager = require('../../../../config/config-manager');

/**
 * Get results from a JMeter test
 * @param {Object} options - Command options
 */
async function results(options) {
    // Check if configuration is set up
    if (!configManager.isConfigured()) {
        console.log(chalk.red('Error: Missing required configuration.'));
        const missing = configManager.getMissingKeys();
        missing.forEach(key => {
            console.log(chalk.yellow(`  - ${key}`));
        });
        console.log(chalk.yellow('\nUse "loadfocus-api config set" to configure these values.'));
        return;
    }

    // Create spinner
    const spinner = ora('Fetching test results...').start();

    try {
        // Create client
        const client = new JMeterClient();

        // Get test results
        const apiResult = await resultsEndpoint(client, {
            testrunname: options.name,
            testrunid: options.id,
            from: options.from ? parseInt(options.from) : undefined,
            to: options.to ? parseInt(options.to) : undefined,
            granularity: options.granularity,
            filter: options.filter // Add filter option
        });

        // Transform the API response to the desired format
        const transformedResults = transformResults(apiResult);

        spinner.succeed('Test results retrieved successfully');

        // Display results header
        console.log();
        console.log(chalk.cyan('Test Results:'));
        console.log(chalk.cyan('------------'));
        console.log(`Test Name: ${chalk.green(options.name)}`);
        console.log(`Test ID: ${chalk.green(options.id)}`);

        // Show filter if provided
        if (options.filter) {
            console.log(`Filter: ${chalk.green(options.filter)}`);
        }

        // Determine which format to display based on the format option
        const format = options.format ? options.format.toLowerCase() : 'summary';

        switch (format) {
        case 'json':
            displayJsonFormat(transformedResults);
            break;
        case 'raw':
            displayRawFormat(apiResult);
            break;
        case 'summary':
        default:
            displayTransformedSummary(transformedResults);
            break;
        }

    } catch (error) {
        spinner.fail('Failed to retrieve test results');

        if (error.data) {
            console.log(chalk.red('Error details:'));
            console.log(JSON.stringify(error.data, null, 2));
        } else {
            console.log(chalk.red(`Error: ${error.message}`));
        }
    }
}

/**
 * Transform API results to the desired format
 * @param {Array} apiResults - The API response array
 * @returns {Object} Transformed results object
 */
function transformResults(apiResults) {
    // If no results, return empty object
    if (!apiResults || apiResults.length === 0) {
        return {
            samples: 0,
            avgresponse: 0,
            errors: 0,
            p99: 0,
            p95: 0,
            max: 0,
            min: 0,
            hitspersec: 0
        };
    }

    // Create summarized object from the first result
    // In a real application, you might want to aggregate multiple results
    const firstResult = apiResults[0];

    return {
        samples: firstResult.statscnt || 0,
        avgresponse: firstResult.mean || 0,
        errors: firstResult.ec || 0,
        p99: firstResult.p99 || 0,
        p95: firstResult.p95 || 0,
        max: firstResult.max || 0,
        min: firstResult.min || 0,
        hitspersec: firstResult.th || 0
    };
}

/**
 * Display the transformed summary information in table format
 * @param {Object} summary - Transformed summary data
 */
function displayTransformedSummary(summary) {
    console.log();
    console.log(chalk.cyan('Summary:'));

    // Create a table for the summary
    const table = new Table({
        head: [chalk.cyan('Metric'), chalk.cyan('Value')],
        colWidths: [20, 20]
    });

    // Add rows for the transformed summary
    table.push(
        ['Samples', formatValue(summary.samples)],
        ['Avg Response Time', `${formatValue(summary.avgresponse)} ms`],
        ['Errors', formatValue(summary.errors)],
        ['99th Percentile', `${formatValue(summary.p99)} ms`],
        ['95th Percentile', `${formatValue(summary.p95)} ms`],
        ['Max Response Time', `${formatValue(summary.max)} ms`],
        ['Min Response Time', `${formatValue(summary.min)} ms`],
        ['Hits Per Second', formatValue(summary.hitspersec)]
    );

    console.log(table.toString());
}

/**
 * Display the transformed results in JSON format
 * @param {Object} summary - Transformed summary data
 */
function displayJsonFormat(summary) {
    console.log();
    console.log(chalk.cyan('JSON Format:'));
    console.log(JSON.stringify(summary, null, 2));
}

/**
 * Display the raw API response
 * @param {Array} apiResults - The API response array
 */
function displayRawFormat(apiResults) {
    console.log();
    console.log(chalk.cyan('Raw Response:'));

    if (!apiResults || apiResults.length === 0) {
        console.log(chalk.yellow('No data available.'));
        return;
    }

    // Determine columns from first data item
    const firstItem = apiResults[0];
    const columns = Object.keys(firstItem);

    // Create table with dynamic columns
    const table = new Table({
        head: columns.map(col => chalk.cyan(formatMetricName(col))),
        colWidths: columns.map(col => getColumnWidth(col, apiResults))
    });

    // Maximum number of rows to display
    const maxRows = 20;
    const displayData = apiResults.length > maxRows ? apiResults.slice(0, maxRows) : apiResults;

    // Add rows
    displayData.forEach(item => {
        const row = columns.map(col => formatMetricValue(col, item[col]));
        table.push(row);
    });

    console.log(table.toString());

    // If we truncated the data, show a message
    if (apiResults.length > maxRows) {
        console.log(chalk.yellow(`\nShowing ${maxRows} of ${apiResults.length} rows. Use JSON format for complete data.`));
    }
}

/**
 * Format value for display
 * @param {number} value - Value to format
 * @returns {string} Formatted value
 */
function formatValue(value) {
    if (value === null || value === undefined) {
        return '-';
    }

    if (typeof value === 'number') {
        // Format number with 2 decimal places if it has decimals
        return Number.isInteger(value) ? value.toString() : value.toFixed(2);
    }

    return value.toString();
}

/**
 * Format metric name for display
 * @param {string} name - Metric name
 * @returns {string} Formatted name
 */
function formatMetricName(name) {
    // Map of common abbreviations to full names
    const nameMap = {
        'p90': 'Percentile 90',
        'p95': 'Percentile 95',
        'p99': 'Percentile 99',
        'mean': 'Mean Response Time',
        'median': 'Median Response Time',
        'min': 'Min Response Time',
        'max': 'Max Response Time',
        'ep': 'Error Percentage',
        'rt': 'Response Time',
        'l': 'Label',
        'ct': 'Content Type',
        'ec': 'Error Count',
        'th': 'Throughput',
        'timestamp': 'Timestamp',
        'statscnt': 'Stats Count',
        'kbrs': 'KB Received/sec',
        'kbss': 'KB Sent/sec',
        'sd': 'Standard Deviation'
    };

    // If we have a mapping, use it, otherwise capitalize the name
    if (nameMap[name]) {
        return nameMap[name];
    } else {
        return name.charAt(0).toUpperCase() + name.slice(1);
    }
}

/**
 * Format metric value for display
 * @param {string} key - Metric key
 * @param {*} value - Metric value
 * @returns {string} Formatted value
 */
function formatMetricValue(key, value) {
    if (value === null || value === undefined) {
        return '-';
    }

    // Format timestamp values
    if (key === 'timestamp' && !isNaN(value)) {
        return new Date(value).toLocaleString();
    }

    // Format time values (in ms)
    if (['mean', 'median', 'min', 'max', 'p90', 'p95', 'p99', 'rt'].includes(key) && !isNaN(value)) {
        return `${value.toFixed(2)} ms`;
    }

    // Format percentage values
    if (['ep'].includes(key) && !isNaN(value)) {
        return `${value.toFixed(2)}%`;
    }

    // Format throughput values
    if (['th'].includes(key) && !isNaN(value)) {
        return `${value.toFixed(2)}/sec`;
    }

    // Format KB values
    if (['kbrs', 'kbss'].includes(key) && !isNaN(value)) {
        return `${value.toFixed(2)} KB/s`;
    }

    // Default formatting
    return value.toString();
}

/**
 * Calculate appropriate column width based on column type and data
 * @param {string} column - Column name
 * @param {Array} data - Result data
 * @returns {number} Column width
 */
function getColumnWidth(column, data) {
    const headerWidth = formatMetricName(column).length;
    let maxDataWidth = 0;

    // Find the maximum width of data in this column
    data.forEach(item => {
        const valueStr = formatMetricValue(column, item[column]);
        maxDataWidth = Math.max(maxDataWidth, valueStr.length);
    });

    // Use the larger of header or data width, plus some padding
    return Math.max(headerWidth, maxDataWidth) + 2;
}

module.exports = results;