/**
 * Map JMeter test results from API format to runTest command format
 * @param {Array} apiResults - Raw API results array
 * @returns {Object} Mapped results in runTest command format
 */
function mapJMeterResults(apiResults) {
    // Validate input
    if (!Array.isArray(apiResults) || apiResults.length === 0) {
        return {
            samples: 0,
            avgresponse: 0,
            errors: 0,
            p99: 0,
            p95: 0,
            max: 0,
            min: 0,
            hitspersec: 0
        };
    }

    // Aggregate results across all entries
    const aggregatedResults = apiResults.reduce((acc, result) => {
        // Accumulate samples
        acc.samples += result.statscnt || result.no || 0;

        // Error tracking
        acc.errors += result.ecnt || result.ec || 0;

        // Response times
        acc.responseTimeEntries.push(result.rt || result.mean || 0);
        acc.maxResponseTimes.push(result.max || 0);
        acc.minResponseTimes.push(result.min || Infinity);

        // Percentile response times
        if (result.p99) acc.p99Entries.push(result.p99);
        if (result.p95) acc.p95Entries.push(result.p95);

        // Throughput
        if (result.th) acc.throughputEntries.push(result.th);

        return acc;
    }, {
        samples: 0,
        errors: 0,
        responseTimeEntries: [],
        maxResponseTimes: [],
        minResponseTimes: [],
        p99Entries: [],
        p95Entries: [],
        throughputEntries: []
    });

    // Calculate final metrics
    return {
        samples: aggregatedResults.samples,
        avgresponse: aggregatedResults.responseTimeEntries.length > 0
            ? calculateMean(aggregatedResults.responseTimeEntries)
            : 0,
        errors: aggregatedResults.errors,
        p99: aggregatedResults.p99Entries.length > 0
            ? calculatePercentile(aggregatedResults.p99Entries)
            : 0,
        p95: aggregatedResults.p95Entries.length > 0
            ? calculatePercentile(aggregatedResults.p95Entries)
            : 0,
        max: Math.max(...aggregatedResults.maxResponseTimes, 0),
        min: Math.min(...aggregatedResults.minResponseTimes, 0),
        hitspersec: aggregatedResults.throughputEntries.length > 0
            ? calculateMean(aggregatedResults.throughputEntries)
            : 0
    };
}

/**
 * Calculate mean of an array of numbers
 * @param {number[]} numbers - Array of numbers
 * @returns {number} Mean value
 */
function calculateMean(numbers) {
    if (numbers.length === 0) return 0;
    const sum = numbers.reduce((a, b) => a + b, 0);
    return Number((sum / numbers.length).toFixed(3));
}

/**
 * Calculate percentile value
 * @param {number[]} numbers - Array of numbers
 * @returns {number} Percentile value
 */
function calculatePercentile(numbers) {
    if (numbers.length === 0) return 0;

    // Sort the numbers
    const sorted = [...numbers].sort((a, b) => a - b);

    // Round to nearest whole number
    return Number(sorted[Math.floor(sorted.length * 0.99)].toFixed(0));
}

module.exports = {
    mapJMeterResults,
    calculateMean,
    calculatePercentile
};