/**
 * JMeter runs command
 */
const chalk = require('chalk');
const ora = require('ora');
const Table = require('cli-table3');
const runsEndpoint = require('../endpoints/runs');
const JMeterClient = require('../client');
const configManager = require('../../../../config/config-manager');

/**
 * Get recent test runs
 * @param {Object} options - Command options
 */
async function runs(options) {
    // Check if configuration is set up
    if (!configManager.isConfigured()) {
        console.log(chalk.red('Error: Missing required configuration.'));
        const missing = configManager.getMissingKeys();
        missing.forEach(key => {
            console.log(chalk.yellow(`  - ${key}`));
        });
        console.log(chalk.yellow('\nUse "loadfocus-api config set" to configure these values.'));
        return;
    }

    // Create spinner
    const spinner = ora('Fetching recent test runs...').start();

    try {
        // Create client
        const client = new JMeterClient();

        // Get recent runs
        const result = await runsEndpoint(client, {
            term: options.term
        });

        spinner.succeed('Test runs retrieved successfully');

        // Display runs
        if (result.length === 0) {
            console.log(chalk.yellow('\nNo test runs found.'));
            return;
        }

        console.log();
        console.log(chalk.cyan(`Recent Test Runs${options.term ? ` matching "${options.term}"` : ''}:`));

        // Create table for results
        const table = new Table({
            head: [
                chalk.cyan('Name'),
                chalk.cyan('ID'),
                chalk.cyan('Status'),
                chalk.cyan('Start Time'),
                chalk.cyan('Duration'),
                chalk.cyan('VUs')
            ],
            colWidths: [30, 20, 15, 20, 15, 10]
        });

        // Add rows
        result.forEach(run => {
            const statusColor = getStatusColor(run.status);

            table.push([
                run.name || '-',
                run.id || '-',
                statusColor(run.status || 'Unknown'),
                run.startTime ? new Date(run.startTime).toLocaleString() : '-',
                run.duration ? formatDuration(run.duration) : '-',
                run.virtualUsers || '-'
            ]);
        });

        console.log(table.toString());

        console.log();
        console.log(chalk.cyan('Next Steps:'));
        console.log(`- Check test status: ${chalk.green('loadfocus-api jmeter status --name "<testname>" --id "<testid>"')}`);
        console.log(`- Get test results: ${chalk.green('loadfocus-api jmeter results --name "<testname>" --id "<testid>"')}`);

    } catch (error) {
        spinner.fail('Failed to retrieve test runs');

        if (error.data) {
            console.log(chalk.red('Error details:'));
            console.log(JSON.stringify(error.data, null, 2));
        } else {
            console.log(chalk.red(`Error: ${error.message}`));
        }
    }
}

/**
 * Get color based on status
 * @param {string} status - Test status
 * @returns {Function} Chalk color function
 */
function getStatusColor(status) {
    if (!status) return chalk.white;

    const statusLower = status.toLowerCase();

    if (statusLower.includes('running') || statusLower.includes('progress')) {
        return chalk.blue;
    } else if (statusLower.includes('complete') || statusLower.includes('success')) {
        return chalk.green;
    } else if (statusLower.includes('fail') || statusLower.includes('error')) {
        return chalk.red;
    } else if (statusLower.includes('queue') || statusLower.includes('pending')) {
        return chalk.yellow;
    }

    return chalk.white;
}

/**
 * Format duration in seconds to human-readable string
 * @param {number} seconds - Duration in seconds
 * @returns {string} Formatted duration
 */
function formatDuration(seconds) {
    if (typeof seconds !== 'number') return '-';

    const hrs = Math.floor(seconds / 3600);
    const mins = Math.floor((seconds % 3600) / 60);
    const secs = seconds % 60;

    let result = '';
    if (hrs > 0) result += `${hrs}h `;
    if (mins > 0) result += `${mins}m `;
    result += `${secs}s`;

    return result.trim();
}

module.exports = runs;