/**
 * JMeter run-test command
 * Executes a test, waits for completion, and evaluates results against thresholds
 */
const chalk = require('chalk');
const ora = require('ora');
const executeEndpoint = require('../endpoints/execute');
const statusEndpoint = require('../endpoints/status');
const resultsEndpoint = require('../endpoints/results');
const JMeterClient = require('../client');
const configManager = require('../../../../config/config-manager');
const { mapJMeterResults } = require('./resultsMapper.js');

/**
 * Run a complete test workflow
 * @param {Object} options - Command options
 */
async function runTest(options) {
    // Parse options with defaults
    const testName = options.name;
    const testId = options.id; // Optional - if provided, will skip execution
    const waitTimeout = parseInt(options.waitTimeout || 1800);
    const pollInterval = parseInt(options.pollInterval || 10) * 1000; // Convert to ms
    const resultsInterval = parseInt(options.resultsInterval || 30);
    const format = options.format || 'pretty';

    // Parse thresholds
    const thresholds = parseThresholds(options);

    // Check if configuration is set up
    if (!configManager.isConfigured()) {
        console.log(chalk.red('Error: Missing required configuration.'));
        const missing = configManager.getMissingKeys();
        missing.forEach(key => {
            console.log(chalk.yellow(`  - ${key}`));
        });
        console.log(chalk.yellow('\nUse "loadfocus-api config set" to configure these values.'));
        return;
    }

    // Create client
    const client = new JMeterClient();

    // Overall results object to track the process
    const runResult = {
        testName,
        testId: null,
        executed: false,
        completed: false,
        startTime: Date.now(),
        endTime: null,
        duration: null,
        finalState: null,
        results: null,
        thresholds: thresholds,
        evaluations: {},
        passed: true, // Will be set to false if any threshold fails
        error: null
    };

    try {
        // Step 1: Execute the test (skip if testId is provided)
        let currentTestId = testId;

        if (!currentTestId) {
            const spinner = ora('Executing JMeter test...').start();

            try {
                const executeResult = await executeEndpoint(client, {
                    testrunname: testName
                });

                currentTestId = executeResult.testId || executeResult.testrunid;

                if (!currentTestId) {
                    throw new Error('Failed to get test ID from execute response');
                }

                spinner.succeed(`Test execution started with ID: ${currentTestId}`);
                runResult.executed = true;
                runResult.testId = currentTestId;
            } catch (error) {
                spinner.fail('Failed to execute test');
                throw error;
            }
        } else {
            console.log(chalk.blue(`Using existing test with ID: ${currentTestId}`));
            runResult.testId = currentTestId;
        }

        // Step 2: Wait for test completion
        console.log(chalk.blue(`\nWaiting for test to complete (timeout: ${waitTimeout}s)...`));

        const startWaitTime = Date.now();
        let lastResultsCheck = 0;

        // eslint-disable-next-line no-constant-condition
        while (true) {
            // Check if we've exceeded the maximum wait time
            let currentWaitTime = Date.now();
            let elapsedSeconds = Math.floor((currentWaitTime - startWaitTime) / 1000);

            
            if (elapsedSeconds >= waitTimeout) {
                throw new Error(`Maximum wait time exceeded (${waitTimeout}s)`);
            }

            // Get current status
            const statusSpinner = ora('Checking test status...').start();
            let statusResult;

            try {
                statusResult = await statusEndpoint(client, {
                    testrunname: testName,
                    testrunid: currentTestId
                });

                const state = statusResult.state;
                statusSpinner.succeed(`Current state: ${state} (elapsed: ${elapsedSeconds}s)`);

                // Check if test has finished
                if (state === 'finished') {
                    console.log(chalk.green('Test completed successfully!'));
                    runResult.completed = true;
                    runResult.finalState = state;
                    break;
                } else if (state === 'failed' || state === 'error') {
                    throw new Error(`Test failed with state: ${state}`);
                }
            } catch (error) {
                statusSpinner.fail('Failed to check test status');
                throw error;
            }

            // Check interim results periodically
            const timeSinceLastResults = elapsedSeconds - lastResultsCheck;
            if (timeSinceLastResults >= resultsInterval) {
                console.log(chalk.blue('\nChecking interim results...'));

                try {
                    const rawResults = await resultsEndpoint(client, {
                        testrunname: testName,
                        testrunid: currentTestId
                    });

                    const interimResults = mapJMeterResults(rawResults);

                    console.log(interimResults);

                    if (interimResults.samples && interimResults.samples > 0) {
                        console.log(chalk.blue('Interim Results:'));
                        console.log(`  Samples: ${chalk.yellow(interimResults.samples)}`);

                        // Show interim metrics and threshold evaluations
                        Object.entries(thresholds).forEach(([metric, threshold]) => {
                            const value = interimResults[metric];
                            if (value !== undefined) {
                                const isWithinThreshold = evaluateThreshold(value, threshold);
                                console.log(`  ${formatMetricName(metric)}: ${chalk.yellow(value)} (Threshold: ${threshold.value} ${threshold.operator}) - ${isWithinThreshold ? chalk.green('OK') : chalk.red('EXCEEDS THRESHOLD')
                                }`);
                            }
                        });
                    } else {
                        console.log(chalk.yellow('No results data available yet'));
                    }
                } catch (error) {
                    console.log(error);
                    console.log(chalk.yellow('Could not retrieve interim results yet'));
                }

                lastResultsCheck = elapsedSeconds;
            }

            // Wait before polling again
            await new Promise(resolve => setTimeout(resolve, pollInterval));
        }

        // Step 3: Get final results
        console.log(chalk.blue('\nGetting final test results...'));
        const resultsSpinner = ora('Fetching results...').start();

        try {
            const rawResults = await resultsEndpoint(client, {
                testrunname: testName,
                testrunid: currentTestId
            });

            const finalResults = mapJMeterResults(rawResults);
            runResult.results = finalResults;
            runResult.endTime = Date.now();
            runResult.duration = Math.floor((runResult.endTime - runResult.startTime) / 1000);

            resultsSpinner.succeed('Test results retrieved successfully');

            // Step 4: Evaluate results against all thresholds
            runResult.passed = true; // Assume passing unless a threshold fails

            // Evaluate each threshold
            Object.entries(thresholds).forEach(([metric, threshold]) => {
                const value = finalResults[metric];

                if (value !== undefined) {
                    const metricPassed = evaluateThreshold(value, threshold);

                    // Record evaluation result
                    runResult.evaluations[metric] = {
                        value: value,
                        threshold: threshold,
                        passed: metricPassed
                    };

                    // If any threshold fails, the overall test fails
                    if (!metricPassed) {
                        runResult.passed = false;
                    }
                }
            });

            // Display overall result
            if (runResult.passed) {
                console.log(chalk.green(`\nTEST PASSED: All metrics within thresholds`));
            } else {
                console.log(chalk.red(`\nTEST FAILED: One or more metrics exceed thresholds`));
            }

            // Display summary
            if (format === 'json') {
                // Return structured JSON result
                console.log(JSON.stringify(runResult, null, 2));
            } else {
                // Display pretty summary
                console.log(chalk.blue('\nTest Summary:'));
                console.log(`Test Name: ${chalk.yellow(testName)}`);
                console.log(`Test ID: ${chalk.yellow(currentTestId)}`);
                console.log(`Duration: ${chalk.yellow(runResult.duration)}s`);
                console.log(`Samples: ${chalk.yellow(finalResults.samples)}`);

                // Display all metrics and their evaluations
                console.log(chalk.blue('\nMetric Evaluations:'));
                Object.entries(runResult.evaluations).forEach(([metric, evaluation]) => {
                    const formattedName = formatMetricName(metric);
                    const formattedValue = formatMetricValue(metric, evaluation.value);
                    const threshold = evaluation.threshold;
                    const passed = evaluation.passed;

                    console.log(`${formattedName}: ${chalk.yellow(formattedValue)} (Threshold: ${threshold.operator} ${threshold.value}) - ${passed ? chalk.green('PASSED') : chalk.red('FAILED')
                    }`);
                });

                console.log(`\nOverall Result: ${runResult.passed ? chalk.green('PASSED') : chalk.red('FAILED')}`);
            }

            // Return non-zero exit code if test failed
            if (!runResult.passed) {
                process.exitCode = 1;
            }
        } catch (error) {
            resultsSpinner.fail('Failed to retrieve test results');
            throw error;
        }
    } catch (error) {
        runResult.error = error.message || 'Unknown error occurred';

        if (format === 'json') {
            // Return structured JSON with error
            console.log(JSON.stringify(runResult, null, 2));
        } else {
            // Display error message
            console.log(chalk.red(`Error: ${error.message}`));
        }

        // Set exit code for CI/CD systems
        process.exitCode = 1;
    }
}

/**
 * Parse threshold options from command line
 * @param {Object} options - Command options
 * @returns {Object} Parsed thresholds map
 */
function parseThresholds(options) {
    const thresholds = {};

    // Legacy support for --threshold option (maps to avgresponse)
    if (options.threshold) {
        thresholds.avgresponse = {
            operator: '<=',
            value: parseFloat(options.threshold)
        };
    }

    // Parse thresholds from the thresholds option if provided
    if (options.thresholds) {
        // Split by comma to get each threshold
        const thresholdStrings = options.thresholds.split(',');

        thresholdStrings.forEach(thresholdStr => {
            // Expected format: metric(operator)value, e.g., "avgresponse<=100" or "errors=0"
            const match = thresholdStr.match(/^([a-z0-9]+)([<>=!]+)([0-9.]+)$/i);

            if (match) {
                const [, metric, operator, valueStr] = match;
                const value = parseFloat(valueStr);

                if (!isNaN(value)) {
                    thresholds[metric] = {
                        operator,
                        value
                    };
                }
            }
        });
    }

    // If no thresholds were specified, use a default avgresponse threshold
    if (Object.keys(thresholds).length === 0) {
        thresholds.avgresponse = {
            operator: '<=',
            value: 100
        };
    }

    return thresholds;
}

/**
 * Evaluate if a value meets a threshold
 * @param {number} value - The metric value to check
 * @param {Object} threshold - Threshold with operator and value
 * @returns {boolean} True if value meets threshold, false otherwise
 */
function evaluateThreshold(value, threshold) {
    switch (threshold.operator) {
    case '<':
        return value < threshold.value;
    case '<=':
        return value <= threshold.value;
    case '>':
        return value > threshold.value;
    case '>=':
        return value >= threshold.value;
    case '=':
    case '==':
        return value === threshold.value;
    case '!=':
        return value !== threshold.value;
    default:
        return false;
    }
}

/**
 * Format metric name for display
 * @param {string} name - Metric name
 * @returns {string} Formatted name
 */
function formatMetricName(name) {
    // Map of common metric names to user-friendly names
    const nameMap = {
        'avgresponse': 'Average Response Time',
        'samples': 'Total Samples',
        'errors': 'Error Count',
        'p95': '95th Percentile',
        'p99': '99th Percentile',
        'max': 'Max Response Time',
        'min': 'Min Response Time',
        'hitspersec': 'Hits Per Second'
    };

    return nameMap[name] || name;
}

/**
 * Format metric value for display
 * @param {string} metric - Metric name
 * @param {number} value - Metric value
 * @returns {string} Formatted value
 */
function formatMetricValue(metric, value) {
    if (value === null || value === undefined) {
        return '-';
    }

    // Time-based metrics (in ms)
    if (['avgresponse', 'p95', 'p99', 'max', 'min'].includes(metric)) {
        return `${value.toFixed(2)} ms`;
    }

    // Throughput
    if (metric === 'hitspersec') {
        return `${value.toFixed(2)}/sec`;
    }

    // Count metrics
    if (['samples', 'errors'].includes(metric)) {
        return value.toString();
    }

    // Default formatting
    return typeof value === 'number' ? value.toFixed(2) : value.toString();
}

module.exports = runTest;