/**
 * JMeter run-tests command
 * Executes multiple tests from a configuration file, either sequentially or in parallel
 */
const chalk = require('chalk');
// ora is not used in this file
const fs = require('fs');
const path = require('path');
const yaml = require('js-yaml');
const runTest = require('./runtest');
// JMeterClient is not used in this file
const configManager = require('../../../../config/config-manager');

/**
 * Run multiple tests based on a configuration file
 * @param {Object} options - Command options
 */
async function runTests(options) {
    // Parse options with defaults
    const configFile = options.config;
    const waitTimeout = parseInt(options.waitTimeout || 1800);
    const pollInterval = parseInt(options.pollInterval || 10);
    const resultsInterval = parseInt(options.resultsInterval || 30);
    const format = options.format || 'pretty';

    // Check if configuration is set up
    if (!configManager.isConfigured()) {
        console.log(chalk.red('Error: Missing required configuration.'));
        const missing = configManager.getMissingKeys();
        missing.forEach(key => {
            console.log(chalk.yellow(`  - ${key}`));
        });
        console.log(chalk.yellow('\nUse "loadfocus-api config set" to configure these values.'));
        return;
    }

    // Load and parse the configuration file
    let config;
    try {
        // Read file content
        const fileContent = fs.readFileSync(path.resolve(configFile), 'utf8');
        
        // Parse based on file extension
        if (configFile.toLowerCase().endsWith('.json')) {
            try {
                config = JSON.parse(fileContent);
            } catch (jsonError) {
                console.log(chalk.red(`Error parsing JSON file: ${jsonError.message}`));
                return;
            }
        } else if (configFile.toLowerCase().endsWith('.yaml') || configFile.toLowerCase().endsWith('.yml')) {
            try {
                // Try to parse as YAML with more strict options
                config = yaml.load(fileContent, {
                    schema: yaml.DEFAULT_SCHEMA,
                    json: false
                });
                
                // If that fails, try to convert to JSON first and then parse
                if (!config) {
                    console.log(chalk.yellow('Warning: Initial YAML parsing returned null, trying alternative parsing method...'));
                    
                    // Convert YAML to JSON format first
                    const jsonContent = JSON.stringify(yaml.load(fileContent, { json: true }));
                    config = JSON.parse(jsonContent);
                }
            } catch (yamlError) {
                // If YAML parsing fails, try to parse it as JSON as a fallback
                console.log(chalk.yellow(`Warning: YAML parsing failed, attempting to parse as JSON: ${yamlError.message}`));
                try {
                    config = JSON.parse(fileContent);
                } catch (fallbackError) {
                    console.log(chalk.red(`Error parsing configuration file: ${yamlError.message}`));
                    
                    // Provide more helpful error information
                    console.log(chalk.yellow('\nPlease check your YAML file for syntax errors. Common issues include:'));
                    console.log(chalk.yellow('- Inconsistent indentation'));
                    console.log(chalk.yellow('- Missing spaces after colons'));
                    console.log(chalk.yellow('- Tabs instead of spaces'));
                    console.log(chalk.yellow('- Special characters in strings'));
                    
                    // Show a simplified example of valid YAML
                    console.log(chalk.yellow('\nExample of valid YAML format:'));
                    console.log(chalk.cyan(`mode: sequential
tests:
  - name: Test1
    thresholds:
      metric1:
        operator: <=
        value: 200`));
                    return;
                }
            }
        } else {
            throw new Error('Unsupported file format. Please use .json, .yaml, or .yml');
        }
    } catch (error) {
        console.log(chalk.red(`Error loading configuration file: ${error.message}`));
        return;
    }

    // Validate configuration
    if (!config) {
        console.log(chalk.red('Error: Failed to parse configuration file.'));
        return;
    }
    
    if (!config.tests || !Array.isArray(config.tests) || config.tests.length === 0) {
        console.log(chalk.red('Error: Configuration file must contain a "tests" array with at least one test.'));
        return;
    }

    // Determine execution mode (sequential or parallel)
    const mode = config.mode || 'sequential';
    if (mode !== 'sequential' && mode !== 'parallel') {
        console.log(chalk.red('Error: Invalid execution mode. Must be "sequential" or "parallel".'));
        return;
    }

    console.log(chalk.blue(`Running tests in ${mode} mode...`));
    console.log(chalk.blue(`Found ${config.tests.length} tests to execute.`));

    // Overall results object
    const results = {
        mode,
        startTime: Date.now(),
        endTime: null,
        duration: null,
        tests: [],
        passed: true // Will be set to false if any test fails
    };

    // Execute tests based on mode
    if (mode === 'sequential') {
        // Run tests one after another
        for (let i = 0; i < config.tests.length; i++) {
            const test = config.tests[i];
            console.log(chalk.blue(`\nRunning test ${i+1}/${config.tests.length}: ${test.name}`));
            
            const testResult = await executeTest(test, {
                waitTimeout,
                pollInterval,
                resultsInterval,
                format
            });
            
            results.tests.push(testResult);
            
            // Update overall pass/fail status - safely check if passed property exists
            if (testResult && testResult.passed === false) {
                results.passed = false;
            }
        }
    } else {
        // Run tests in parallel
        console.log(chalk.blue('Starting all tests in parallel...'));
        
        const testPromises = config.tests.map(test => 
            executeTest(test, {
                waitTimeout,
                pollInterval,
                resultsInterval,
                format
            })
        );
        
        const testResults = await Promise.all(testPromises);
        results.tests = testResults;
        
        // Update overall pass/fail status - safely check each result
        results.passed = testResults.every(result => result && result.passed !== false);
    }

    // Calculate overall duration
    results.endTime = Date.now();
    results.duration = Math.floor((results.endTime - results.startTime) / 1000);

    // Display summary
    console.log(chalk.blue('\n========================================'));
    console.log(chalk.blue('Test Execution Summary'));
    console.log(chalk.blue('========================================'));
    console.log(`Mode: ${chalk.yellow(mode)}`);
    console.log(`Total Duration: ${chalk.yellow(results.duration)}s`);
    console.log(`Overall Status: ${results.passed ? chalk.green('PASSED') : chalk.red('FAILED')}`);
    console.log(chalk.blue('\nTest Results:'));
    
    results.tests.forEach((test, index) => {
        if (!test) {
            console.log(`\n${index + 1}. ${chalk.bold('Unknown Test')}: ${chalk.red('ERROR - No test result')}`);
            return;
        }
        
        console.log(`\n${index + 1}. ${chalk.bold(test.testName || 'Unknown Test')}: ${test.passed === false ? chalk.red('FAILED') : (test.passed === true ? chalk.green('PASSED') : chalk.yellow('UNKNOWN'))}`);
        console.log(`   Duration: ${chalk.yellow(test.duration || 'N/A')}s`);
        
        if (test.evaluations && Object.keys(test.evaluations).length > 0) {
            console.log('   Thresholds:');
            Object.entries(test.evaluations).forEach(([metric, evaluation]) => {
                const { value, threshold, passed } = evaluation;
                console.log(`     ${formatMetricName(metric)}: ${chalk.yellow(value)} (${threshold.operator} ${threshold.value}) - ${passed ? chalk.green('PASSED') : chalk.red('FAILED')}`);
            });
        }
        
        if (test.error) {
            console.log(`   Error: ${chalk.red(test.error)}`);
        }
    });

    // Output final status
    console.log(chalk.blue('\n========================================'));
    console.log(`Overall Status: ${results.passed ? chalk.green('PASSED') : chalk.red('FAILED')}`);
    
    // If format is json, output the full results object
    if (format === 'json') {
        console.log(JSON.stringify(results, null, 2));
    }

    return results;
}

/**
 * Execute a single test with thresholds
 * @param {Object} test - Test configuration
 * @param {Object} options - Execution options
 * @returns {Object} Test result
 */
async function executeTest(test, options) {
    try {
        // Prepare options for runTest
        const testOptions = {
            name: test.name,
            waitTimeout: options.waitTimeout,
            pollInterval: options.pollInterval,
            resultsInterval: options.resultsInterval,
            format: options.format
        };

        // Convert thresholds from configuration format to command-line format
        if (test.thresholds) {
            const thresholdStrings = [];
            Object.entries(test.thresholds).forEach(([metric, threshold]) => {
                thresholdStrings.push(`${metric}${threshold.operator}${threshold.value}`);
            });
            testOptions.thresholds = thresholdStrings.join(',');
        }

        // Run the test using the existing runTest command
        const result = await runTest(testOptions);
        
        // Ensure the result has the expected properties
        return {
            testName: test.name,
            testId: result?.testId,
            passed: result?.passed === false ? false : (result?.passed === true ? true : undefined),
            executed: result?.executed,
            completed: result?.completed,
            startTime: result?.startTime,
            endTime: result?.endTime,
            duration: result?.duration,
            finalState: result?.finalState,
            results: result?.results,
            thresholds: result?.thresholds,
            evaluations: result?.evaluations || {},
            error: result?.error
        };
    } catch (error) {
        // Return error result
        return {
            testName: test.name,
            passed: false,
            error: error.message,
            startTime: Date.now(),
            endTime: Date.now(),
            duration: 0
        };
    }
}

/**
 * Format metric name for display
 * @param {string} name - Metric name
 * @returns {string} Formatted name
 */
function formatMetricName(name) {
    const formattedNames = {
        'avgresponse': 'Avg Response Time',
        'minresponse': 'Min Response Time',
        'maxresponse': 'Max Response Time',
        'p90': '90th Percentile',
        'p95': '95th Percentile',
        'p99': '99th Percentile',
        'errors': 'Error Count',
        'errorrate': 'Error Rate',
        'throughput': 'Throughput',
        'hitspersec': 'Hits Per Second',
        'samples': 'Sample Count'
    };

    return formattedNames[name] || name;
}

module.exports = runTests;
