/**
 * JMeter status command
 */
const chalk = require('chalk');
const ora = require('ora');
const statusEndpoint = require('../endpoints/status');
const JMeterClient = require('../client');
const configManager = require('../../../../config/config-manager');

/**
 * Get the status of a JMeter test
 * @param {Object} options - Command options
 */
async function status(options) {
    // Check if configuration is set up
    if (!configManager.isConfigured()) {
        console.log(chalk.red('Error: Missing required configuration.'));
        const missing = configManager.getMissingKeys();
        missing.forEach(key => {
            console.log(chalk.yellow(`  - ${key}`));
        });
        console.log(chalk.yellow('\nUse "loadfocus-api config set" to configure these values.'));
        return;
    }

    // Create spinner
    const spinner = ora('Fetching test status...').start();

    try {
        // Create client
        const client = new JMeterClient();

        // Get test status
        const result = await statusEndpoint(client, {
            testrunname: options.name,
            testrunid: options.id
        });

        spinner.succeed('Test status retrieved successfully');

        // Determine which format to display based on the format option
        const format = options.format ? options.format.toLowerCase() : 'pretty';

        switch (format) {
        case 'json':
            displayJsonFormat(result, options);
            break;
        case 'raw':
            displayRawFormat(result);
            break;
        case 'pretty':
        default:
            displayPrettyFormat(result, options);
            break;
        }

    } catch (error) {
        spinner.fail('Failed to retrieve test status');

        if (error.data) {
            console.log(chalk.red('Error details:'));
            console.log(JSON.stringify(error.data, null, 2));
        } else {
            console.log(chalk.red(`Error: ${error.message}`));
        }
    }
}

/**
 * Display status in pretty format with colors and formatting
 * @param {Object} result - Status response
 * @param {Object} options - Command options
 */
function displayPrettyFormat(result, options) {
    // Display status details
    console.log();
    console.log(chalk.cyan('Test Status:'));
    console.log(chalk.cyan('------------'));
    console.log(`Test Name: ${chalk.green(options.name)}`);
    console.log(`Test ID: ${chalk.green(result.testrunid)}`);

    // Show state information
    if (result.state) {
        const statusColor = getStatusColor(result.state);
        console.log(`Status: ${statusColor(result.state)}`);
    }

    // Display time information
    if (result.teststarttime) {
        const startTime = new Date(parseInt(result.teststarttime));
        console.log(`Start Time: ${chalk.green(startTime.toLocaleString())}`);
    }

    if (result.teststoptime) {
        const stopTime = new Date(parseInt(result.teststoptime));
        console.log(`End Time: ${chalk.green(stopTime.toLocaleString())}`);

        // Calculate and show duration if both start and stop times are available
        if (result.teststarttime) {
            const durationMs = parseInt(result.teststoptime) - parseInt(result.teststarttime);
            const durationSec = Math.floor(durationMs / 1000);
            console.log(`Duration: ${chalk.green(formatDuration(durationSec))}`);
        }
    }

    // Show machine information
    if (result.machines && result.machines.length > 0) {
        console.log();
        console.log(chalk.cyan('Machine Information:'));

        result.machines.forEach((machine, index) => {
            console.log(`Machine ${index + 1}:`);
            console.log(`  Region: ${chalk.green(machine.cloudmachineregion || 'Not specified')}`);

            const machineState = machine.state;
            if (machineState) {
                const machineStateColor = getStatusColor(machineState);
                console.log(`  State: ${machineStateColor(machineState)}`);
            }

            // Show machine-specific time details
            if (machine.teststarttime && machine.teststoptime) {
                const machineDurationMs = parseInt(machine.teststoptime) - parseInt(machine.teststarttime);
                const machineDurationSec = Math.floor(machineDurationMs / 1000);
                console.log(`  Duration: ${chalk.green(formatDuration(machineDurationSec))}`);
            }
        });
    }

    // Show test type and number of machines
    console.log();
    console.log(chalk.cyan('Test Details:'));
    console.log(`Test Type: ${chalk.green(result.testtype || 'Not specified')}`);
    console.log(`Number of Machines: ${chalk.green(result.numberofmachines || 0)}`);

    // Show additional messages if available
    if (result.message) {
        console.log();
        console.log(chalk.cyan('Message:'));
        console.log(chalk.yellow(result.message));
    }

    console.log();
    console.log(chalk.cyan('Next Steps:'));
    console.log(`- Get test results: ${chalk.green(`loadfocus-api jmeter results --name "${options.name}" --id "${result.testrunid}"`)}`);
}

/**
 * Display status in JSON format
 * @param {Object} result - Status response
 * @param {Object} options - Command options
 */
function displayJsonFormat(result, options) {
    // Create a simplified status object
    const statusSummary = {
        testName: options.name,
        testId: result.testrunid,
        state: result.state,
        startTime: result.teststarttime ? parseInt(result.teststarttime) : null,
        endTime: result.teststoptime ? parseInt(result.teststoptime) : null,
        duration: result.teststarttime && result.teststoptime ?
            Math.floor((parseInt(result.teststoptime) - parseInt(result.teststarttime)) / 1000) : null,
        testType: result.testtype,
        numberOfMachines: result.numberofmachines,
        team: result.team_id,
        message: result.message
    };

    // If there are machines, include their information
    if (result.machines && result.machines.length > 0) {
        statusSummary.machines = result.machines.map(machine => ({
            region: machine.cloudmachineregion,
            state: machine.state,
            startTime: machine.teststarttime ? parseInt(machine.teststarttime) : null,
            endTime: machine.teststoptime ? parseInt(machine.teststoptime) : null,
            duration: machine.teststarttime && machine.teststoptime ?
                Math.floor((parseInt(machine.teststoptime) - parseInt(machine.teststarttime)) / 1000) : null
        }));
    }

    console.log();
    console.log(JSON.stringify(statusSummary, null, 2));
}

/**
 * Format the raw API response for display
 * @param {Object} result - Status response
 */
function displayRawFormat(result) {
    console.log(JSON.stringify(result, null, 2));
}

/**
 * Get color based on status
 * @param {string} status - Test status
 * @returns {Function} Chalk color function
 */
function getStatusColor(status) {
    const statusLower = status.toLowerCase();

    if (statusLower.includes('running') || statusLower.includes('progress') || statusLower.includes('pending')) {
        return chalk.blue;
    } else if (statusLower.includes('complete') || statusLower.includes('success') || statusLower.includes('finished')) {
        return chalk.green;
    } else if (statusLower.includes('fail') || statusLower.includes('error')) {
        return chalk.red;
    } else if (statusLower.includes('queue') || statusLower.includes('waiting')) {
        return chalk.yellow;
    }

    return chalk.white;
}

/**
 * Format duration in seconds to human-readable string
 * @param {number} seconds - Duration in seconds
 * @returns {string} Formatted duration
 */
function formatDuration(seconds) {
    const hrs = Math.floor(seconds / 3600);
    const mins = Math.floor((seconds % 3600) / 60);
    const secs = seconds % 60;

    let result = '';
    if (hrs > 0) result += `${hrs}h `;
    if (mins > 0) result += `${mins}m `;
    result += `${secs}s`;

    return result.trim();
}

module.exports = status;