/**
 * Configuration command
 */
const chalk = require('chalk');
const Table = require('cli-table3');
const configManager = require('../../config/config-manager');

/**
 * Handle the config command
 * @param {Object} options - Command options
 * @param {Object} command - Commander command object
 */
function configCommand(options, command) {
    // Determine the subcommand
    const subCommand = command.args[0];

    if (subCommand === 'set') {
        // Setting configuration values
        const configValues = {
            apiKey: options.apikey,
            teamId: options.teamid,
            apiUrl: options.url
        };

        configManager.set(configValues);

        console.log(chalk.green('Configuration updated successfully!'));

        // Show the new configuration
        showConfig();
    } else if (subCommand === 'show') {
        // Show current configuration
        showConfig();
    } else {
        console.log(chalk.yellow('Please specify a subcommand: set or show'));
        console.log('Example:');
        console.log('  loadfocus-api config set --apikey YOUR_API_KEY --teamid YOUR_TEAM_ID');
        console.log('  loadfocus-api config show');
    }
}

/**
 * Display the current configuration
 */
function showConfig() {
    const config = configManager.getAll();

    // Create a table for nicer display
    const table = new Table({
        head: [chalk.cyan('Setting'), chalk.cyan('Value')],
        colWidths: [20, 60]
    });

    // Add rows for each config value
    for (const [key, value] of Object.entries(config)) {
        // Mask API key for security
        const displayValue = key === 'apiKey'
            ? value.substring(0, 4) + '...' + value.substring(value.length - 4)
            : value;

        table.push([key, displayValue]);
    }

    console.log(table.toString());

    // Check if all required configs are set
    if (!configManager.isConfigured()) {
        const missing = configManager.getMissingKeys();
        console.log(chalk.yellow('\nWarning: Missing required configuration:'));
        missing.forEach(key => {
            console.log(chalk.yellow(`  - ${key}`));
        });
        console.log(chalk.yellow('\nUse "loadfocus-api config set" to configure these values.'));
    }
}

module.exports = configCommand;