const fs = require('fs');
const path = require('path');
const os = require('os');

class ConfigManager {
    constructor() {
        this.configDir = path.join(os.homedir(), '.loadfocus-api');
        this.configPath = path.join(this.configDir, 'config.json');
        this.requiredKeys = ['apiKey', 'teamId'];
        this.defaults = {
            apiUrl: 'https://loadfocus.com' // Production API URL
        };

        // Ensure config directory exists
        this._ensureConfigDir();

        // Load existing config or create empty config
        this._loadConfig();
    }

    _ensureConfigDir() {
        if (!fs.existsSync(this.configDir)) {
            fs.mkdirSync(this.configDir, { recursive: true });
        }
    }

    _loadConfig() {
        if (!fs.existsSync(this.configPath)) {
            this.config = {};
            this._saveConfig();
        } else {
            try {
                const configData = fs.readFileSync(this.configPath, 'utf8');
                this.config = JSON.parse(configData);
            } catch (error) {
                console.error(`Error loading config: ${error.message}`);
                this.config = {};
                this._saveConfig();
            }
        }
    }

    _saveConfig() {
        try {
            fs.writeFileSync(this.configPath, JSON.stringify(this.config, null, 2), 'utf8');
        } catch (error) {
            console.error(`Error saving config: ${error.message}`);
        }
    }

    set(config) {
        for (const key in config) {
            if (config[key] !== undefined) {
                this.config[key] = config[key];
            }
        }
        this._saveConfig();
    }

    get(key, defaultValue) {
        return this.config[key] || this.defaults[key] || defaultValue;
    }

    getAll() {
        const result = { ...this.config };
        // Add defaults for any missing keys
        for (const key in this.defaults) {
            if (!result[key]) {
                result[key] = this.defaults[key];
            }
        }
        return result;
    }

    delete(key) {
        if (this.config[key]) {
            delete this.config[key];
            this._saveConfig();
        }
    }

    clear() {
        this.config = {};
        this._saveConfig();
    }

    isConfigured() {
        return this.requiredKeys.every(key => Boolean(this.get(key)));
    }

    getMissingKeys() {
        return this.requiredKeys.filter(key => !this.get(key));
    }
}

// Export a singleton instance
module.exports = new ConfigManager();